==> fuc.js-0.1.7.tgz: package/components/elements/paging/paging.js <==
import $ from 'jquery';

const re = /{{([\w-]+)}}/g;

function applyTemplate(template, values) {
  return template.replace(re, function (m, name) {
    return values[name] !== undefined ? values[name] : '';
  });
}

// 改变选项时，是否重新渲染
const reloadOption = ['totalCount', 'totalPages', 'pageNum', 'pageSize', 'rowList',
  'condition', 'jsonReader',
];

const Paging = (($) => {
  const NAME = 'paging';
  const DATA_KEY = 'fuc.paging';
  const EVENT_KEY = `.${DATA_KEY}`;
  const DATA_API_KEY = '.data-api';
  const JQUERY_NO_CONFLICT = $.fn[NAME];

  const ClassName = {
    PAGING_ITEM: 'paging-item',
    DISABLED: 'disabled',
    ACTIVE: 'active',
  };

  const Selector = {};

  const Event = {
    CLICK_DATA_API: `click${EVENT_KEY}${DATA_API_KEY}`,
    CHANGE_DATA_API: `change${EVENT_KEY}${DATA_API_KEY}`,
  };

  class Paging {
    /**
     * pagingControl 控制是否显示页码、总记录数、切换每页条数和跳转至某一页 {Boolean}
     * recordPerPage 是否显示【切换每页条数】{Boolean}
     * jumpControl 是否显示跳转至某一页 {Boolean}
     * @type {{pagingControl: boolean, recordPerPage: boolean, jumpControl: boolean}}
     */
    static defaultConfig = {
      pagingControl: true, // 控制是否显示页码、总记录数、切换每页条数和跳转至某一页
      records: true, // 是否显示页码和记录数
      recordPerPage: true, // 是否显示【切换每页条数】
      jumpControl: true, // 是否显示跳转至某一页
      isRenderPagingBeforData: true, // 是否在加载数据之前渲染分页
      direction: 'right', // 默认右对齐
      totalCount: 0, // 总记录数
      totalPages: 0, // 总页码 | 如果设置了loadPageData，不用传，直接取后台返回的 totalPages 即可
      pageNum: 1, // 当前第几页
      pageSize: 10, // 每页显示的记录数
      rowList: [5, 10, 20, 50, 100], // 可切换每页显示页码列表值
      condition: {}, // 数据请求条件
      jsonReader: { // 后台返回数据格式对应字段
        items: 'items',
        pageNum: 'pageNum',
        totalPages: 'totalPages',
        totalCount: 'totalCount',
      },
      pagingControlTmpl: { // 分页渲染页码、总记录数、切换分页条数模板
        pagingDirection: '<div class="fuc-paging-{{direction}}">{{pagingItems}}</div>',
        controlPanel: '<ul class="fuc-paging-control">{{controlPanel}}</ul>',
        recordHtml: '<li class="fuc-paging-control-item">共{{totalPages}}页{{totalCount}}条记录</li>',
        recordPerPage: `<li class="fuc-paging-control-item">
            <div class="fuc-select" data-toggle="select">
              <div class="fuc-select-title">
                <input type="text"/>
              </div>
              <ul class="fuc-select-options">
                {{selectOptions}}
              </ul>
            </div></li>`,
        jumpControl: `<li class="fuc-paging-control-item">
          跳至
          <div class="fuc-input-inline input">
            <div class="fuc-input-group">
              <input type="text" class="fuc-input-control js-jump-input" value="{{jumpInputValue}}"/>
            </div>
          </div>
          页
          <button class="fuc-btn fuc-m-l-1 primary js-jump-btn">确定</button>
        </li>`,
      },
      separator: '', // 显示页码总记录数、切换每页条数和跳转至某一页中间分隔符
    };

    constructor(element, config) {
      this.element = element;
      const _config = {...Paging.defaultConfig, ...config};
      // 设置 config 到 this 上
      Object.keys(_config).forEach((key) => {
        this[key] = _config[key];
      });

      this._loadData();
      this._addEventListeners();
    }

    // 设置监听事件
    _addEventListeners() {
      $(this.element).on(Event.CLICK_DATA_API, '.fuc-pagination li a', this._handleSwitchPage);
      $(this.element).on(Event.CLICK_DATA_API, '.js-jump-btn', this._handleJumpPage);
    }

    _handleSwitchPage = (evt) => {
      evt.preventDefault();
      const {currentTarget} = evt;
      const pageNum = parseInt($(currentTarget).parent().data('pagenum'), 10);
      this._handleChangePage(pageNum);
    };

    _handleJumpPage = (evt) => {
      evt.preventDefault();
      const $input = $(this.element).find('.js-jump-input');
      const pageNum = parseInt($input.val(), 10);
      if (!Number.isNaN(pageNum)) {
        this._handleChangePage(pageNum);
      }
    };

    _handleChangePerPage = (evt) => {
      const {value} = evt;

      const pageSize = parseInt(value, 10);
      this.pageSize = pageSize;
      this._loadData();
    };

    _handleChangePage(pageNum) {
      if (this.totalPages === 1) {
        return;
      }
      if (pageNum <= 1) {
        pageNum = 1;
      }

      if (pageNum >= this.totalPages) {
        pageNum = this.totalPages;
      }

      this.pageNum = pageNum;
      this._loadData();
    }

    _loadData() {
      const {loadPageData, pageSize, pageNum, jsonReader} = this;

      // 渲染数据
      if (loadPageData && typeof loadPageData === 'function') {
        if (this.isRenderPagingBeforData) {
          this._renderPaging();
        }

        loadPageData({
          pageNum,
          pageSize,
          condition: this.condition || {},
          callback: (data) => {
            this.totalPages = data[jsonReader.totalPages];
            this.totalCount = data[jsonReader.totalCount];
            this._renderPaging();
          },
        });
      } else {
        this._renderPaging();
      }
    }

    _renderPaging() {
      const pageArray = this._calculatePage();
      const {pagingDirection} = this.pagingControlTmpl;

      let html = '<ul class="fuc-pagination fuc-pagination-control">';
      html += `<li class="${this.pageNum === 1 ? `${ClassName.DISABLED}` : ''}" data-pagenum="${this.pageNum - 1}">${this.pageNum === 1 ? '<span>上一页</span>' : '<a href="#">上一页</a>'}</li>`;
      const pageItems = pageArray.map((item, index) => {
        if (item === '...') {
          return '<li class="fuc-paging-more"><span>...</span></li>';
        }
        return `<li class="${this.pageNum === item ? `${ClassName.ACTIVE}` : ''}" data-pagenum="${item}">${this.pageNum === item ? `<span>${item}</span>` : `<a href="#">${item}</a>`}</li>`;

      });
      html += pageItems.join('');
      /*eslint-disable max-len*/
      html += `<li class="${this.pageNum === this.totalPages ? `${ClassName.DISABLED}` : ''}" data-pagenum="${this.pageNum + 1}">${this.pageNum === this.totalPages ? '<span>下一页</span>' : '<a href="#">下一页</a>'}</li>`;
      html += '</ul>';

      if (this.pagingControl) {
        html += this._renderPagingControl();
      }

      html = applyTemplate(pagingDirection, {
        direction: this.direction,
        pagingItems: html,
      });

      $(this.element).html(html);

      // 注册 select 事件
      $(this.element).find('[data-toggle="select"]').select().on('selected.fuc.select', this._handleChangePerPage);
    }

    _renderPagingControl() {
      const {controlPanel, recordHtml, recordPerPage, jumpControl} = this.pagingControlTmpl;

      let pageHtml = [];

      if (this.records) {
        pageHtml.push(applyTemplate(recordHtml, {
          totalPages: this.totalPages,
          totalCount: this.totalCount,
        }));
      }

      if (this.recordPerPage) {
        pageHtml.push(applyTemplate(recordPerPage, {
          selectOptions: this.rowList.map((item) => {
            return `<li class="fuc-select-option${this.pageSize === item ? ' active' : ''}" data-value="${item}">
                    <a class="fuc-select-text" href="#">${item}条/页</a>
              </li>`;
          }).join(''),
        }));
      }

      if (this.jumpControl) {
        pageHtml.push(applyTemplate(jumpControl, {
          jumpInputValue: this.pageNum,
        }));
      }

      pageHtml = pageHtml.join(pageHtml.length > 1 ? this.separator : '');

      pageHtml = applyTemplate(controlPanel, {
        controlPanel: pageHtml,
      });

      return pageHtml;
    }

    /**
     * 计算页码显示算法，返回一个页码数组
     * @returns {Array}
     */
    _calculatePage() {
      const totalPages = this.totalPages;
      const pageNum = this.pageNum;

      const pageArray = [];
      if (totalPages < 8) {
        for (let i = 1; i <= totalPages; i++) {
          pageArray.push(i);
        }
      } else {
        pageArray.push(1);
        if (pageNum > 4) {
          pageArray.push('...');
        }

        if (pageNum < 4) {
          for (let i = 2; i <= 6; i++) {
            pageArray.push(i);
          }
        } else if (pageNum >= 4 && totalPages - pageNum >= 3) {
          for (let i = pageNum - 2; i <= pageNum + 2; i++) {
            pageArray.push(i);
          }
        } else {
          for (let i = totalPages - 4; i < totalPages; i++) {
            pageArray.push(i);
          }
        }

        //总页码 - 当前页 大于 3 显示
        if (totalPages - pageNum > 3) {
          pageArray.push('...');
        }
        pageArray.push(totalPages);
      }

      return pageArray;
    }

    // Public

    // 重新渲染分页
    reload(condition) {
      this.condition = condition;
      if (condition.pageNum) {
        this.pageNum = condition.pageNum;
      }
      delete this.condition.pageNum;

      this._loadData();
    }

    /**
     * 动态修改配置项
     * @param options {object}
     */
    setOptions(options) {
      if (typeof options !== 'object') {
        throw new TypeError('The params options should be an object');
      }
      let isReload = false;
      // 设置 options 到 this 上
      Object.keys(options).forEach((key) => {
        this[key] = options[key];
        if (reloadOption.indexOf(key) !== -1) {
          isReload = true;
        }
      });
      if (isReload) {
        this._loadData();
      } else {
        this._renderPaging();
      }
    }

    // Static
    static _jQueryInterface(config, params) {
      return this.each(function () {
        let data = $(this).data(DATA_KEY);

        if (!data) {
          data = new Paging(this, config);
          $(this).data(DATA_KEY, data);
        }

        if (typeof config === 'string') {
          if (typeof data[config] === 'undefined') {
            throw new TypeError(`No method named "${config}"`);
          }
          data[config](params);
        }
      });
    }
  }

  /**
   * ------------------------------------------------------------------------
   * jQuery
   * ------------------------------------------------------------------------
   */

  $.fn[NAME] = Paging._jQueryInterface;
  $.fn[NAME].Constructor = Paging;
  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Paging._jQueryInterface;
  };

  return Paging;
})($);

export default Paging;

==> fuc.js-0.2.0.tgz: package/components/elements/paging/paging.js <==
import $ from 'jquery';

const re = /{{([\w-]+)}}/g;

function applyTemplate(template, values) {
  return template.replace(re, function (m, name) {
    return values[name] !== undefined ? values[name] : '';
  });
}

// 改变选项时，是否重新渲染
const reloadOption = ['totalCount', 'totalPages', 'pageNum', 'pageSize', 'rowList',
  'condition', 'jsonReader',
];

const Paging = (($) => {
  const NAME = 'paging';
  const DATA_KEY = 'fuc.paging';
  const EVENT_KEY = `.${DATA_KEY}`;
  const DATA_API_KEY = '.data-api';
  const JQUERY_NO_CONFLICT = $.fn[NAME];

  const ClassName = {
    PAGING_ITEM: 'paging-item',
    DISABLED: 'disabled',
    ACTIVE: 'active',
  };

  const Selector = {};

  const Event = {
    CLICK_DATA_API: `click${EVENT_KEY}${DATA_API_KEY}`,
    CHANGE_DATA_API: `change${EVENT_KEY}${DATA_API_KEY}`,
  };

  class Paging {
    /**
     * pagingControl 控制是否显示页码、总记录数、切换每页条数和跳转至某一页 {Boolean}
     * recordPerPage 是否显示【切换每页条数】{Boolean}
     * jumpControl 是否显示跳转至某一页 {Boolean}
     * @type {{pagingControl: boolean, recordPerPage: boolean, jumpControl: boolean}}
     */
    static defaultConfig = {
      pagingControl: true, // 控制是否显示页码、总记录数、切换每页条数和跳转至某一页
      records: true, // 是否显示页码和记录数
      recordPerPage: true, // 是否显示【切换每页条数】
      jumpControl: true, // 是否显示跳转至某一页
      isRenderPagingBeforData: true, // 是否在加载数据之前渲染分页
      direction: 'right', // 默认右对齐
      totalCount: 0, // 总记录数
      totalPages: 0, // 总页码 | 如果设置了loadPageData，不用传，直接取后台返回的 totalPages 即可
      pageNum: 1, // 当前第几页
      pageSize: 10, // 每页显示的记录数
      rowList: [5, 10, 20, 50, 100], // 可切换每页显示页码列表值
      condition: {}, // 数据请求条件
      jsonReader: { // 后台返回数据格式对应字段
        items: 'items',
        pageNum: 'pageNum',
        totalPages: 'totalPages',
        totalCount: 'totalCount',
      },
      pagingControlTmpl: { // 分页渲染页码、总记录数、切换分页条数模板
        pagingDirection: '<div class="fuc-paging-{{direction}}">{{pagingItems}}</div>',
        controlPanel: '<ul class="fuc-paging-control">{{controlPanel}}</ul>',
        recordHtml: '<li class="fuc-paging-control-item">共{{totalPages}}页{{totalCount}}条记录</li>',
        recordPerPage: `<li class="fuc-paging-control-item">
            <div class="fuc-select" data-toggle="select">
              <div class="fuc-select-title">
                <input type="text"/>
              </div>
              <ul class="fuc-select-options">
                {{selectOptions}}
              </ul>
            </div></li>`,
        jumpControl: `<li class="fuc-paging-control-item">
          跳至
          <div class="fuc-input-inline input">
            <div class="fuc-input-group">
              <input type="text" class="fuc-input-control js-jump-input" value="{{jumpInputValue}}"/>
            </div>
          </div>
          页
          <button class="fuc-btn fuc-m-l-1 primary js-jump-btn">确定</button>
        </li>`,
      },
      separator: '', // 显示页码总记录数、切换每页条数和跳转至某一页中间分隔符
    };

    constructor(element, config) {
      this.element = element;
      const _config = {...Paging.defaultConfig, ...config};
      // 设置 config 到 this 上
      Object.keys(_config).forEach((key) => {
        this[key] = _config[key];
      });

      this._loadData();
      this._addEventListeners();
    }

    // 设置监听事件
    _addEventListeners() {
      $(this.element).on(Event.CLICK_DATA_API, '.fuc-pagination li a', this._handleSwitchPage);
      $(this.element).on(Event.CLICK_DATA_API, '.js-jump-btn', this._handleJumpPage);
    }

    _handleSwitchPage = (evt) => {
      evt.preventDefault();
      const {currentTarget} = evt;
      const pageNum = parseInt($(currentTarget).parent().data('pagenum'), 10);
      this._handleChangePage(pageNum);
    };

    _handleJumpPage = (evt) => {
      evt.preventDefault();
      const $input = $(this.element).find('.js-jump-input');
      const pageNum = parseInt($input.val(), 10);
      if (!Number.isNaN(pageNum)) {
        this._handleChangePage(pageNum);
      }
    };

    _handleChangePerPage = (evt) => {
      const {value} = evt;

      const pageSize = parseInt(value, 10);
      this.pageSize = pageSize;
      this._loadData();
    };

    _handleChangePage(pageNum) {
      if (this.totalPages === 1) {
        return;
      }
      if (pageNum <= 1) {
        pageNum = 1;
      }

      if (pageNum >= this.totalPages) {
        pageNum = this.totalPages;
      }

      this.pageNum = pageNum;
      this._loadData();
    }

    _loadData() {
      const {loadPageData, pageSize, pageNum, jsonReader} = this;

      // 渲染数据
      if (loadPageData && typeof loadPageData === 'function') {
        if (this.isRenderPagingBeforData) {
          this._renderPaging();
        }

        loadPageData({
          pageNum,
          pageSize,
          condition: this.condition || {},
          callback: (data) => {
            this.totalPages = data[jsonReader.totalPages];
            this.totalCount = data[jsonReader.totalCount];
            this._renderPaging();
          },
        });
      } else {
        this._renderPaging();
      }
    }

    _renderPaging() {
      const pageArray = this._calculatePage();
      const {pagingDirection} = this.pagingControlTmpl;

      let html = '<ul class="fuc-pagination fuc-pagination-control">';
      html += `<li class="${this.pageNum === 1 ? `${ClassName.DISABLED}` : ''}" data-pagenum="${this.pageNum - 1}">${this.pageNum === 1 ? '<span>上一页</span>' : '<a href="#">上一页</a>'}</li>`;
      const pageItems = pageArray.map((item, index) => {
        if (item === '...') {
          return '<li class="fuc-paging-more"><span>...</span></li>';
        }
        return `<li class="${this.pageNum === item ? `${ClassName.ACTIVE}` : ''}" data-pagenum="${item}">${this.pageNum === item ? `<span>${item}</span>` : `<a href="#">${item}</a>`}</li>`;

      });
      html += pageItems.join('');
      /*eslint-disable max-len*/
      html += `<li class="${this.pageNum === this.totalPages ? `${ClassName.DISABLED}` : ''}" data-pagenum="${this.pageNum + 1}">${this.pageNum === this.totalPages ? '<span>下一页</span>' : '<a href="#">下一页</a>'}</li>`;
      html += '</ul>';

      if (this.pagingControl) {
        html += this._renderPagingControl();
      }

      html = applyTemplate(pagingDirection, {
        direction: this.direction,
        pagingItems: html,
      });

      $(this.element).html(html);

      // 注册 select 事件
      $(this.element).find('[data-toggle="select"]').select().on('selected.fuc.select', this._handleChangePerPage);
    }

    _renderPagingControl() {
      const {controlPanel, recordHtml, recordPerPage, jumpControl} = this.pagingControlTmpl;

      let pageHtml = [];

      if (this.records) {
        pageHtml.push(applyTemplate(recordHtml, {
          totalPages: this.totalPages,
          totalCount: this.totalCount,
        }));
      }

      if (this.recordPerPage) {
        pageHtml.push(applyTemplate(recordPerPage, {
          selectOptions: this.rowList.map((item) => {
            return `<li class="fuc-select-option${this.pageSize === item ? ' active' : ''}" data-value="${item}">
                    <a class="fuc-select-text" href="#">${item}条/页</a>
              </li>`;
          }).join(''),
        }));
      }

      if (this.jumpControl) {
        pageHtml.push(applyTemplate(jumpControl, {
          jumpInputValue: this.pageNum,
        }));
      }

      pageHtml = pageHtml.join(pageHtml.length > 1 ? this.separator : '');

      pageHtml = applyTemplate(controlPanel, {
        controlPanel: pageHtml,
      });

      return pageHtml;
    }

    /**
     * 计算页码显示算法，返回一个页码数组
     * @returns {Array}
     */
    _calculatePage() {
      const totalPages = this.totalPages;
      const pageNum = this.pageNum;

      const pageArray = [];
      if (totalPages < 8) {
        for (let i = 1; i <= totalPages; i++) {
          pageArray.push(i);
        }
      } else {
        pageArray.push(1);
        if (pageNum > 4) {
          pageArray.push('...');
        }

        if (pageNum < 4) {
          for (let i = 2; i <= 6; i++) {
            pageArray.push(i);
          }
        } else if (pageNum >= 4 && totalPages - pageNum >= 3) {
          for (let i = pageNum - 2; i <= pageNum + 2; i++) {
            pageArray.push(i);
          }
        } else {
          for (let i = totalPages - 4; i < totalPages; i++) {
            pageArray.push(i);
          }
        }

        //总页码 - 当前页 大于 3 显示
        if (totalPages - pageNum > 3) {
          pageArray.push('...');
        }
        pageArray.push(totalPages);
      }

      return pageArray;
    }

    // Public

    /**
     * 重新渲染分页
     * @param condition
     * @param resetFirstPageNum 是否重置为第一页
     */
    reload(condition, resetFirstPageNum = true) {
      this.condition = {...condition};
      if (resetFirstPageNum) {
        this.pageNum = 1;
      } else if (condition.pageNum !== undefined) {
        this.pageNum = condition.pageNum;
        delete this.condition.pageNum;
      }

      this._loadData();
    }

    /**
     * 动态修改配置项
     * @param options {object}
     */
    setOptions(options) {
      if (typeof options !== 'object') {
        throw new TypeError('The params options should be an object');
      }
      let isReload = false;
      // 设置 options 到 this 上
      Object.keys(options).forEach((key) => {
        this[key] = options[key];
        if (reloadOption.indexOf(key) !== -1) {
          isReload = true;
        }
      });
      if (isReload) {
        this._loadData();
      } else {
        this._renderPaging();
      }
    }

    // Static
    static _jQueryInterface(config, ...params) {
      return this.each(function () {
        let data = $(this).data(DATA_KEY);

        if (!data) {
          data = new Paging(this, config);
          $(this).data(DATA_KEY, data);
        }

        if (typeof config === 'string') {
          if (typeof data[config] === 'undefined') {
            throw new TypeError(`No method named "${config}"`);
          }
          data[config](...params);
        }
      });
    }
  }

  /**
   * ------------------------------------------------------------------------
   * jQuery
   * ------------------------------------------------------------------------
   */

  $.fn[NAME] = Paging._jQueryInterface;
  $.fn[NAME].Constructor = Paging;
  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Paging._jQueryInterface;
  };

  return Paging;
})($);

export default Paging;
